"""
Default constant values across whole SDK
"""


# Text colors on terminal
class TerminalColor:
    BOLD = '\033[1m'
    ENDC = '\033[0m'
    FAIL = '\033[91m'
    OK = '\033[92m'
    WARN = '\033[93m'

# Podonos homepage
PODONOS_HOME = 'https://www.podonos.com/'

# Podonos docs API key
PODONOS_DOCS_API_KEY = 'https://docs.podonos.com/apikey'

# API key environment variable
PODONOS_API_KEY = 'PODONOS_API_KEY'

# Podonos Workspace
PODONOS_WORKSPACE = "https://workspace.podonos.com"

# Podonos API base URL
PODONOS_API_BASE_URL = "https://prod.podonosapi.com"

# Contact email
PODONOS_CONTACT_EMAIL = "hello@podonos.com"
